//Script1
//The following script was run on an entire folder of 96 deconvolved multi-tiff images of 30 slices each
//The script can also be run on non deconvovled stacks with minor modifications
//Main functions of the script include preprocessing and assigning of pronephric areas to the images
dir =getDirectory("Choose a directory");
setBatchMode(true);
function main(dir)												// creates output folders to save resulting csv and images files
{
      print("Running entire process on: "+dir);

     // STEP 0: CROP
      cropDir = dir + "Crop/";
      File.makeDirectory(cropDir);
      step0_crop(dir,cropDir);

      // STEP 1: RECROP
      recropDir = cropDir + "Recrop/";
      File.makeDirectory(recropDir);
      step1_recrop(cropDir,recropDir);

      // STEP 2.1: CYST DETECTION
      cystDir = recropDir + "Cyst/";
      File.makeDirectory(cystDir);
      step2a_cysts(recropDir,cystDir);

      // STEP 2.2: KIDNEY_DETECTION
      kidneyDir = recropDir + "Kidney/";
      File.makeDirectory(kidneyDir);
      step2b_kidneys(recropDir,kidneyDir);
}
function closeAll()											// close all open windows
{
  run("Close All");
  windows = getList("window.titles");
  for ( i=0; i < windows.length; i++ )
  {
    window = windows[i];
	selectWindow(window);
	run("Close");
  }
}
/** Goes through all the multilayer tif files, z-projects and crops it and saves the output to respective folder
 */
function step0_crop( dir, outDir )
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      print("    Cropping: "+file);
      open(dir+file);
      crop();
      saveAs("tiff", outDir+file);
      closeAll();
    }
  }
}

/** Auto crops the image to minimize the black border and then cuts off the
 *  lower part of the image and saves the output to respective folder
 */
function step1_recrop( dir, outDir )
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      open(dir+file);
      recrop();
      saveAs("tiff", outDir+file);
      closeAll();
    }
  }
}
/** Finds all the potential cyst "particle" areas and saves the output to respective folder
 */
function step2a_cysts( dir, outDir )
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      open(dir+file);
      Title = File.nameWithoutExtension();
      detectCysts();
      saveAs("tiff", outDir+file);
      saveAs("Measurements", outDir + Title + ".csv");
      closeAll();
    }
  }
}
/** Finds all the potential kidney "particle" areas and saves the output to respective folder
 */
function step2b_kidneys( dir, outDir )
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      open(dir+file);
      Title = File.nameWithoutExtension();
      detectKidneys();
      saveAs("tiff", outDir+file);
      saveAs("Measurements", outDir + Title + ".csv");
      closeAll();
    }
  }
}

setBatchMode(true);

function crop() 
{
  sliceCount = nSlices();

  run("Rotate 90 Degrees Right");

  sourceWindow = getTitle();
  run("Find focused slices", "select=100 variance=0.000");
  label = getMetadata("Label"); // get the subtitle

  l1 = lengthOf("Z_"); 
  l2 = indexOf(label,"Z_");

  str1 = substring(label, l1+l2); 
  str2 = substring(label, 0, l2);
  focus = parseInt(str1+str2); // convert string into integer, fs: focussed slide

  print("    Focussed Slice: " + focus);

  slice_first= focus - 4; 
  slice_last = focus + 3;
  if( slice_first< 1         ) slice_first= 1;
  if( slice_last > sliceCount) slice_last = sliceCount;
  selectWindow(sourceWindow);
  run("Make Substack...", "  slices=" + slice_first + "-" + slice_last);
  substackWindow = getTitle();
  run("Duplicate...", "duplicate");
  run("Remove Outliers...", "radius=1 threshold=50 which=Bright stack");
  run("Z Project...", "projection=[Max Intensity]");
  run("Canvas Size...", "width=512 height=512 position=Center");

}
function recrop()
{
  run("Duplicate...", " ");
  run("Auto Threshold", "method=Li white");
  run("Canvas Size...", "width=" + getWidth() + " height=" + getHeight()/1.6 + " position=Center");
  run("Remove Outliers...", "radius=6 threshold=50 which=Bright stack");
}
function detectCysts()
{
  run("Canvas Size...", "width=" + getWidth()/1.11 + " height=" + getHeight()/1.38+ " position=Bottom-Center");
  run("Canvas Size...", "width=" + getWidth()/1.09 + " height=" + getHeight()+ " position=Bottom-Right");
  run("Remove Outliers...", "radius=22 threshold=50 which=Bright");
  run("Invert");
  run("Clear Results");
  run("Set Measurements...", "area mean standard modal min centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction redirect=None decimal=3");
  run("Convex Hull Plus", "mode=[Convex Hull selection] log");
  if (isOpen("Exception")) { 
  	selectWindow("Exception");
    run("Close");  }
  run("Analyze Particles...", "  show=Outlines display include");
}
function detectKidneys()
{
  run("Canvas Size...", "width=" + getWidth()/1.11 + " height=" + getHeight()/1.38+ " position=Bottom-Center");
  run("Clear Results");
  run("Set Measurements...", "area mean standard modal min centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction redirect=None decimal=3");
  //Creates an empty row in results table in case of blank well with no particles
  setResult("Area",nResults,"0");setResult("Mean",0,"0");setResult("StdDev",0,"0");setResult("Mode",0,"0");setResult("Min",0,"0");setResult("Max",0,"0");setResult("X",0,"0");setResult("Y",0,"0");setResult("XM",0,"0");setResult("YM",0,"0");setResult("Perim.",0,"0");setResult("BX",0,"0");setResult("BY",0,"0");
  setResult("Width",nResults,"0");setResult("Height",nResults,"0");setResult("Major",nResults,"0");setResult("Minor",nResults,"0");setResult("Angle",nResults,"0");setResult("Circ.",nResults,"0");setResult("Feret",nResults,"0");setResult("IntDen",nResults,"0");setResult("Median",nResults,"0");setResult("Skew",nResults,"0");setResult("Kurt",nResults,"0");
  setResult("%Area",nResults,"0");setResult("RawIntDen",nResults,"0");setResult("FeretX",nResults,"0");setResult("FeretY",nResults,"0");setResult("FeretAngle",nResults,"0");setResult("MinFeret",nResults,"0");setResult("AR",nResults,"0");setResult("Round",nResults,"0");setResult("Solidity",nResults,"0");
  run("Analyze Particles...", "  show=Outlines display include");
}

main(dir);

//Blank Detection
//The following script was run on an entire folder of 96 preprocessed images obtained from crop folder of script1 to find blank images
setBatchMode(true);
dir = getDirectory("Choose a Directory");
list = getFileList(dir); 
dirName = File.getName(dir); 
outDir = dir+File.separator+"Recrop"+File.separator+"/BlankDetection"+File.separator;  // create output folder to save resulting csv and images files
File.makeDirectory(outDir);
function cyst()																	//fucntion execution for filter application and variance measurement
{
run("Clear Results");
TargetTitle= File.nameWithoutExtension();
run("8-bit");
run("Canvas Size...", "width=512 height=320 position=Bottom-Center");
run("Canvas Size...", "width=412 height=200 position=Top-Center");
run("Laplace Filter");
run("8-bit");
run("Clear Results");
run("Set Measurements...", "  mean redirect=None decimal=5");
run("Measure");
mean = getResult("Mean");
selectWindow("Results");
run("Close");
W = getWidth();
H = getHeight();
b = 0;
Var = 0; 																		// Set to 0 which is out of focus
for (j=0; j<H; j++) {
   for (i=0; i<W; i++) {
      p = getPixel(i,j);
      t = (p-mean)*(p-mean);
      b += t;
      }
   }
Var = b/(H*W); 																	// Maximum value is best-focused, decreasing as defocus increases
print(Var);
}
for (i=0;i<list.length;i++) 													// saving resulting images and csv files in the output folder
{
	  showProgress(i+1, list.length);
      if (endsWith(list[i],"tif") || endsWith(list[i],"TIF"))
      {
      	print(dir+list[i]);
		open(dir+list[i]); 
        cyst();
		TargetTitle= File.nameWithoutExtension();
		saveAs("TIFF", outDir+TargetTitle);
	    selectWindow("Log");
	    saveAs("Text", outDir+TargetTitle+".csv");
}
} 


//Blur Detection
//The following script was run on an entire folder of 96 preprocessed images obtained from crop folder of script1 to find blurry images
setBatchMode(true);
dir = getDirectory("Choose a Directory");
list = getFileList(dir); 
dirName = File.getName(dir); 
outDir = dir+File.separator+"Recrop"+File.separator+"/BlurDetection"+File.separator;		// create output folder to save resulting csv and images files
File.makeDirectory(outDir);																	
function cyst()																//fucntion execution for background subtraction and variance measurement
{
run("Clear Results");
TargetTitle= File.nameWithoutExtension();
run("8-bit");
run("Canvas Size...", "width=512 height=320 position=Bottom-Center");
run("Canvas Size...", "width=412 height=200 position=Top-Center");
run("Unsharp Mask...", "radius=10 mask=0.90");
run("Subtract Background...", "rolling=5");
run("Clear Results");
run("Set Measurements...", "  mean redirect=None decimal=5");
run("Measure");
mean = getResult("Mean");
selectWindow("Results");
run("Close");
W = getWidth();
H = getHeight();
b = 0;
Var = 0; 																	// Set to 0 which is out of focus
for (j=0; j<H; j++) {
   for (i=0; i<W; i++) {
      p = getPixel(i,j);
      t = (p-mean)*(p-mean);
      b += t;
      }
   }
Var = b/(H*W); 																// Maximum value is best-focused, decreasing as defocus increases
print(Var);

}
for (i=0;i<list.length;i++)                            						// saving resulting images and csv files in the output folder
{
	  showProgress(i+1, list.length);
      if (endsWith(list[i],"tif") || endsWith(list[i],"TIF"))
      {
      	print(dir+list[i]);
		open(dir+list[i]); 
        cyst();
		TargetTitle= File.nameWithoutExtension();
		saveAs("TIFF", outDir+TargetTitle);
	    selectWindow("Log");
	    saveAs("Text", outDir+TargetTitle+".csv");	    
}
} 


//Wild type/Cyst Detection
//The following script was run on an entire folder of 96 preprocessed images obtained from crop folder of script1 to detect the presence of wild type/cystic images
dir =getDirectory("Choose a directory");
setBatchMode(true);
function main(dir)
{
      print("Running entire process on: "+dir);                  		// create output folders to save resulting csv and images files

      // STEP 1: Wt_Preprocess
      Wt_preprocessDir = dir + "Wt_preprocess/";
      File.makeDirectory(Wt_preprocessDir);
      step1_Wt_preprocess(dir,Wt_preprocessDir);

      // STEP 2: Wt_FilePlot
      Wt_FilePlotDir = Wt_preprocessDir + "Wt_FilePlot/";
      File.makeDirectory(Wt_FilePlotDir);
      step2_Wt_FilePlot(Wt_preprocessDir,Wt_FilePlotDir);
}
function closeAll()                   									// close all open windows
{
  run("Close All");
  windows = getList("window.titles");
  for ( i=0; i < windows.length; i++ )
  {
    window = windows[i];
	selectWindow(window);
	run("Close");
  }
}
function step1_Wt_preprocess(dir, outDir)
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      print("    Wt_Preprocess: "+file);
      open(dir+file);
      Wt_preprocess();
      saveAs("tiff", outDir+file);
    }
  }
}

function step2_Wt_FilePlot( dir, outDir )
{
  files = getFileList(dir);
  for( i=0; i < files.length; i++ )
  {
    file = files[i];
    if(
         endsWith(file,".tif")
      || endsWith(file,".tiff")
    )
    {
      open(dir+file);
      Wt_FilePlot();
      closeAll();      
    }
  }
}

setBatchMode(true);                  									
function Wt_preprocess() 												// image preprocessing
{
	run("8-bit");
	run("Canvas Size...", "width=512 height=320 position=Bottom-Center");
	run("Canvas Size...", "width=412 height=200 position=Top-Center");
	run("Gaussian Blur...", "sigma=15");
	setAutoThreshold("Li dark");
	run("Convert to Mask");
}
function Wt_FilePlot()
{
	TargetTitle= File.nameWithoutExtension();
	run("Clear Results");
	run("Measure");
	WholeImage_Area = getResult("%Area",0);								// internal blank image check
	if (WholeImage_Area == 100)
	{
	run("Invert");
	run("Select All");
	profile = getProfile(); 
	run("Clear Results");
	saveAs("TIFF", outDir+TargetTitle+"plot");
	run("Clear Results");
	profile = getProfile(); 
	for (i=0; i<profile.length; i++) {
	setResult("Value", i, profile[i]);
	updateResults();
	saveAs("Measurements",  outDir+TargetTitle+".csv");
	}}
	else {																// if not a blank image wild type image check performed
	run("Create Selection");											// enclosing pronephros in bounding box of optimal size
	run("Make Inverse");
	run("To Bounding Box");
	run("Clear Results");
	run("Set Measurements...", "mean standard modal centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction redirect=None decimal=5");
	run("Measure");
	     boundingbox_width = getResult("Width",0);
	     boundingbox_widthNew = boundingbox_width-2;
		 boundingbox_height= getResult("Height",0);
		 boundingbox_heightNew = 98;
		 boundingbox_bx = getResult("BX",0);
		 boundingbox_by = getResult("BY",0);
	    if (boundingbox_height > 98)
	{
		arguments = "width="+boundingbox_widthNew+" height="+boundingbox_heightNew+" x="+boundingbox_bx +" y="+boundingbox_by+ " scaled";
		run("Specify...", arguments);
	}
profile = getProfile(); 												// running plot profile 
		run("Clear Results");
		saveAs("TIFF", outDir+TargetTitle+"plot");
		run("Clear Results");
		profile = getProfile(); 
		for (i=0; i<profile.length; i++) {
		setResult("Value", i, profile[i]);
		updateResults();
		saveAs("Measurements",  outDir+TargetTitle+".csv");				// saving resulting images and csv files in the output folder
	} }
}
main(dir);


//Wild type/Cyst Detection re-check
//The following script was run on an entire folder of 96 preprocessed images obtained from crop folder of script1 to re-check the preclassified wild type/cystic images
setBatchMode(true);
dir = getDirectory("Choose a Directory");
list = getFileList(dir); 
dirName = File.getName(dir); 
outDir = dir+"/MajorCheck"+File.separator;
File.makeDirectory(outDir);										// create output folder to save resulting csv and images files
function cyst()
{
TargetTitle= File.nameWithoutExtension();						// execution of function to process and draw ellipse on pronephros
  run("Clear Results");
  run("Set Measurements...", "area mean standard modal min centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction redirect=None decimal=3");
  run("Convex Hull Plus", "mode=[Convex Hull selection] log");
  if (isOpen("Exception")) { 
  	selectWindow("Exception");
    run("Close");  }
    run("Analyze Particles...", "  show=Ellipses display include");
}
for (i=0;i<list.length;i++) 									// saving resulting images and csv files in the output folder
{
	  showProgress(i+1, list.length);
      if (endsWith(list[i],"tif") || endsWith(list[i],"TIF"))
      {
      	print(dir+list[i]);
		open(dir+list[i]); 
        cyst();
		TargetTitle= File.nameWithoutExtension();
		saveAs("TIFF", outDir+TargetTitle);
	    saveAs("Results", outDir+TargetTitle+".csv");
	  }
} 