/* this set of macros can be used to carry out Smart Imaging applications on an ACQUIFER Imaging Machine IM04. 
 *  The first macro "SmartImaging_400_11" watches a directory reported by the IM04 to D:\\IMAGING-DATA\\JOBS\\Image_Location.pth. Each image is then processed by the second macro "centerofmass_400_11", which detects the center of mass after thresholding and generates a IM04 job file with script commands for altering the screening run.
 */


macro "SmartImaging_400_11" {

setBatchMode (true);

filestring=File.openAsString("D:\\IMAGING-DATA\\JOBS\\Image_Location.pth");
rows=split(filestring, "\n");
directory=replace(rows[0], "\\\\","\\\\\\\\")+"\\\\";
print (directory);

File.makeDirectory(""+directory+"dest");

// get the initial list (without opening the files) 
fulllist = getFileList(directory);
Array.sort(fulllist); 

//push only images into the list array
list = newArray();
for (jj=0; jj<fulllist.length; jj++){
	if (endsWith (fulllist[jj], ".tif")){
		list = Array.concat(fulllist[jj], list);
	}
}

Array.sort(list); 
Array.print(list);

//execute command for each image file
for (ii=0; ii<list.length; ii++){
	 if (endsWith (list[ii], ".tif")){
                        	print (list[ii]);
                                open(directory + list[ii]);
                                selectWindow(list[ii]);
                                run("centerofmass_400_11"); 
                                close();
	 }
}

//watch folder
for (;;) { 
        previousList = list; 
        
        fulllist = getFileList(directory); 
        //push only images into the list array
        list = newArray();
		for (jjj=0; jjj<fulllist.length; jjj++){
			if (endsWith (fulllist[jjj], ".tif")){
				list = Array.concat(fulllist[jjj], list);
			}
		}

       
        if (previousList.length != list.length) { 
        	//sort both lists
                Array.sort(list); 
		Array.sort(previousList);
                // walk both lists 
                j = 0; 
                for (i = 0; i < list.length; i++) {  
                        stopJ = previousList.length; 
                        while (j < stopJ) 	     
                        if (previousList[j] < list[i]) {
                                	j++; 
                                }else{ 
                                	stopJ = j; 
                                }    
                                  
                        if (j >= previousList.length && endsWith (list[i], ".tif")){
                        		//wait(200);  
                                open(directory + list[i]);
                                selectWindow(list[i]);
                                run("centerofmass_400_11"); 
                                close();
                        } else if (list[i] != previousList[j] && endsWith (list[i], ".tif")) {
                        		//wait(200); 
                                open(directory + list[i]);
                                selectWindow(list[i]);
                                run("centerofmass_400_11");  
                                close();
                        }
                } 
        } 

        wait(1000); // wait a second 
} 

}



macro "centerofmass_400_11"{
run("Set Measurements...", "center redirect=None decimal=0");
run("Properties...", "channels=1 slices=1 frames=1 unit=pixel pixel_width=1.0000 pixel_height=1.0000 voxel_depth=1.0000 frame=[0 sec] origin=0,0");

a = getInfo("image.filename");
print (a);
dir = getInfo("image.directory");
print (dir);
File.makeDirectory(""+dir+"dest");
setBatchMode(true);

	run("Duplicate...", "title=b duplicate range=1-130");
	run("Invert");
	run("8-bit");
	run("Auto Threshold", "method=Otsu ignore_white");
	run("Select None");

	getPixelSize(unit, pixelWidth, pixelHeight);
	x=newArray(1); y=newArray(1);
	run("Measure");
	x[0] = round(getResult("XM")/pixelWidth);
	y[0] = round(getResult("YM")/pixelHeight);

	selectWindow(a);
	makeRectangle(x[0]-256, y[0]-256, 512, 512);
	run("Crop");
	saveAs("Tiff", ""+dir+"\\dest\\cropped--"+a+"");
	
	selectWindow("b");
	close();

	f = File.open("D:\\IMAGING-DATA\\JOBS\\"+a+".lck");


	newName = replace (a, "--", "#");
	name = split(newName, "#");
	Array.print(name);
	//Array.print(name);
	

	XLabel = split(name[9], "X");
	YLabel = split(name[10], "Y");
	
	XAbs = XLabel[0];
	YAbs = YLabel[0];
	
	print ("X: "+XAbs+", Y: "+YAbs+"");
	
	offsetX = (x[0]-1024)*(6.5/4); //set to (6.5/2) for 2x
	print ("offsetX :"+offsetX+"");
	offsetY = (1024-y[0])*(6.5/4);//set to (6.5/2) for 2x
	print ("offsetY :"+offsetY+"");
	
	NewXCoord = (round((XAbs)+(offsetX)))/1000;
	NewYCoord = (round((YAbs)+(offsetY)))/1000;
	
	print ("New X: "+NewXCoord+"");
	print ("New Y: "+NewYCoord+"");
	
	WellNo = split(name[13], "WE");
	WellNoCleaned = split(WellNo[0], ".tif");
	print(f, "SetWellNo("+WellNoCleaned[0]+")");
		
	WellCoordinate = split(name[0], "-");
	print(f, "SetMeta('Well Coordinate','"+WellCoordinate[0]+"')");
	
	
	SubPos = split(name[1], "SU");
	print(f, "SetMeta('Well Pos in Well','"+SubPos[0]+"')");
	print(f, "GotoXY("+NewXCoord+","+NewYCoord+",'Abs')");
	print(f, "SetObjective(3)");

	
	/////10x///////
	print(f, "SetLight(3,10,10,0.000)");
	print(f, "AcquireAutofocus(7,100.000,2,'True','True')");
	print(f, "SetLight(3,10,10,0.000,'Flash')");
	print(f, "AcquireAutofocus(6,20.000,2,'True','True')");
	print(f, "SetLight(3,100,20,0.000)");
	print(f, "Acquire(30,4.000,1,'D:\\IMAGING-DATA\\IMAGES\\SCRIPTS','True','SIdata')"); // 30 slices 4um
	print(f, "SetLight(6,60,10,0.000)");
	print(f, "Acquire(30,4.000,1,'D:\\IMAGING-DATA\\IMAGES\\SCRIPTS','True','SIdata')"); // 30 slices 4um
	print(f, "SetLight(0,0,0,0)");
	print(f, "SetObjective(2)"); //set to SetObjective(1) for 2x

	
	File.close(f);
	File.rename("D:\\IMAGING-DATA\\JOBS\\"+a+".lck", "D:\\IMAGING-DATA\\JOBS\\"+a+".job"); 

}